@@ -29,8 +29,10 @@
 #include <QDBusError>
 #include <QDBusMetaType>
 #include <QDBusPendingReply>
+#include <QDBusReply>
 #include <QFile>
 #include <QIcon>
+#include <QStringBuilder>
 
 #include <KIO/OpenUrlJob>
 #include <KLocalizedString>
@@ -41,10 +43,13 @@
 #include <KUser>
 #include <KWallet>
 #include <KWindowSystem>
+#include <KX11Extras>
 
 #include <nm-client.h>
+#include <QCoroCore>
+#include <QCoroDBus>
 
-#define AGENT_SERVICE "org.kde.kded5"
+#define AGENT_SERVICE "org.kde.kded6"
 #define AGENT_PATH "/modules/networkmanagement"
 #define AGENT_IFACE "org.kde.plasmanetworkmanagement"
 
@@ -80,12 +85,17 @@ Handler::Handler(QObject *parent)
 Handler::~Handler() = default;
 
 void Handler::activateConnection(const QString &connection, const QString &device, const QString &specificObject)
+{
+    activateConnectionInternal(connection, device, specificObject);
+}
+
+QCoro::Task<void> Handler::activateConnectionInternal(const QString &connection, const QString &device, const QString &specificObject)
 {
     NetworkManager::Connection::Ptr con = NetworkManager::findConnection(connection);
 
     if (!con) {
         qCWarning(PLASMA_NM_LIBS_LOG) << "Not possible to activate this connection";
-        return;
+        co_return;
     }
 
     if (con->settings()->connectionType() == NetworkManager::ConnectionSettings::Vpn) {
@@ -99,7 +109,7 @@ void Handler::activateConnection(const QString &connection, const QString &devic
                 return md.value(QStringLiteral("X-NetworkManager-Services")) == vpnSetting->serviceType();
             };
 
-            const QVector<KPluginMetaData> plasmaNmPlugins = KPluginMetaData::findPlugins(QStringLiteral("plasma/network/vpn"), filter);
+            const QList<KPluginMetaData> plasmaNmPlugins = KPluginMetaData::findPlugins(QStringLiteral("plasma/network/vpn"), filter);
 
             const QString pluginBaseName = vpnSetting->serviceType().remove(QLatin1String("org.freedesktop.NetworkManager."));
 
@@ -110,8 +120,9 @@ void Handler::activateConnection(const QString &connection, const QString &devic
                 notification->setTitle(con->name());
                 notification->setText(i18n("Plasma is missing support for '%1' VPN connections.", pluginBaseName));
                 notification->setIconName(QStringLiteral("dialog-error"));
-                notification->setActions({i18n("Report Bug")});
-                connect(notification, &KNotification::action1Activated, this, [notification] {
+
+                auto reportBugAction = notification->addAction(i18n("Report Bug"));
+                connect(reportBugAction, &KNotificationAction::activated, this, [notification] {
                     auto *job = new KIO::OpenUrlJob(QUrl(KOSRelease().bugReportUrl()));
                     job->setStartupId(notification->xdgActivationToken().toUtf8());
                     job->start();
@@ -119,7 +130,7 @@ void Handler::activateConnection(const QString &connection, const QString &devic
 
                 notification->sendEvent();
 
-                return;
+                co_return;
             }
 
             // Check missing NetworkManager VPN plugin
@@ -135,16 +146,16 @@ void Handler::activateConnection(const QString &connection, const QString &devic
                 notification->setTitle(con->name());
                 notification->setText(i18n("NetworkManager is missing support for '%1' VPN connections.", pluginBaseName));
                 notification->setIconName(QStringLiteral("dialog-error"));
-                notification->setActions({i18n("Install")});
 
-                connect(notification, &KNotification::action1Activated, this, [notification, pluginBaseName] {
+                auto installAction = notification->addAction(i18n("Install"));
+                connect(installAction, &KNotificationAction::activated, this, [notification, pluginBaseName] {
                     auto *job = new KIO::OpenUrlJob(QUrl(QStringLiteral("appstream:network-manager-") + pluginBaseName));
                     job->setStartupId(notification->xdgActivationToken().toUtf8());
                     job->start();
                 });
 
                 notification->sendEvent();
-                return;
+                co_return;
             }
         }
     }
@@ -167,20 +178,26 @@ void Handler::activateConnection(const QString &connection, const QString &devic
                     m_tmpConnectionPath = connection;
                     m_tmpDevicePath = device;
                     m_tmpSpecificPath = specificObject;
-                    return;
+                    co_return;
                 }
             }
         }
     }
 
-    QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::activateConnection(connection, device, specificObject);
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", Handler::ActivateConnection);
-    watcher->setProperty("connection", con->name());
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    QDBusReply<QDBusObjectPath> reply = co_await NetworkManager::activateConnection(connection, device, specificObject);
+
+    if (!reply.isValid()) {
+        QString error = reply.error().message();
+        KNotification *notification = new KNotification(QStringLiteral("FailedToActivateConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to activate %1", con->name()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(error);
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    }
 }
 
-void Handler::requestWifiCode(const QString &connectionPath, const QString &ssid, int _securityType, const QString &connectionName)
+void Handler::requestWifiCode(const QString &connectionPath, const QString &ssid, int _securityType)
 {
     if (!m_requestWifiCodeWatcher.isNull()) {
         delete m_requestWifiCodeWatcher;
@@ -209,14 +226,14 @@ void Handler::requestWifiCode(const QString &connectionPath, const QString &ssid
         case NetworkManager::Wpa2Eap:
         case NetworkManager::Wpa3SuiteB192:
         case NetworkManager::Leap:
-            Q_EMIT wifiCodeReceived(QString(), connectionName);
+            Q_EMIT wifiCodeReceived(QString(), ssid);
             return;
         }
     }
 
     NetworkManager::Connection::Ptr connection = NetworkManager::findConnection(connectionPath);
     if (!connection) {
-        Q_EMIT wifiCodeReceived(QString(), connectionName);
+        Q_EMIT wifiCodeReceived(QString(), ssid);
         return;
     }
 
@@ -226,11 +243,16 @@ void Handler::requestWifiCode(const QString &connectionPath, const QString &ssid
     m_requestWifiCodeWatcher->setProperty("key", key);
     m_requestWifiCodeWatcher->setProperty("ret", ret);
     m_requestWifiCodeWatcher->setProperty("securityType", static_cast<int>(securityType));
-    m_requestWifiCodeWatcher->setProperty("connectionName", connectionName);
+    m_requestWifiCodeWatcher->setProperty("ssid", ssid);
     connect(m_requestWifiCodeWatcher, &QDBusPendingCallWatcher::finished, this, &Handler::slotRequestWifiCode);
 }
 
-void Handler::addAndActivateConnection(const QString &device, const QString &specificObject, const QString &password)
+void Handler::addAndActivateConnection(const QString &device, const QString &specificParameter, const QString &password)
+{
+    addAndActivateConnectionInternal(device, specificParameter, password);
+}
+
+QCoro::Task<void> Handler::addAndActivateConnectionInternal(const QString &device, const QString &specificObject, const QString &password)
 {
     NetworkManager::AccessPoint::Ptr ap;
     NetworkManager::WirelessDevice::Ptr wifiDev;
@@ -245,7 +267,7 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
     }
 
     if (!ap) {
-        return;
+        co_return;
     }
 
     NetworkManager::ConnectionSettings::Ptr settings =
@@ -300,8 +322,11 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
         QPointer<ConnectionEditorDialog> editor = new ConnectionEditorDialog(settings);
         editor->setAttribute(Qt::WA_DeleteOnClose);
         editor->show();
-        KWindowSystem::setState(editor->winId(), NET::KeepAbove);
-        KWindowSystem::forceActiveWindow(editor->winId());
+
+        if (KWindowSystem::isPlatformX11()) {
+            KX11Extras::setState(editor->winId(), NET::KeepAbove);
+        }
+
         connect(editor.data(), &ConnectionEditorDialog::accepted, [editor, device, specificObject, this]() { //
             addAndActivateConnectionDBus(editor->setting(), device, specificObject);
         });
@@ -327,13 +352,27 @@ void Handler::addAndActivateConnection(const QString &device, const QString &spe
     settings.clear();
 }
 
-void Handler::addConnection(const NMVariantMapMap &map)
+QCoro::Task<void> Handler::addConnection(const NMVariantMapMap &map)
 {
-    QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::addConnection(map);
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", AddConnection);
-    watcher->setProperty("connection", map.value(QStringLiteral("connection")).value(QStringLiteral("id")));
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    const QString connectionId = map.value(QStringLiteral("connection")).value(QStringLiteral("id")).toString();
+
+    QDBusReply<QDBusObjectPath> reply = co_await NetworkManager::addConnection(map);
+
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToAddConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to add connection %1", connectionId));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    } else {
+        KNotification *notification = new KNotification(QStringLiteral("ConnectionAdded"), KNotification::CloseOnTimeout, this);
+        notification->setText(i18n("Connection %1 has been added", connectionId));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setTitle(connectionId);
+        notification->setIconName(QStringLiteral("dialog-information"));
+        notification->sendEvent();
+    }
 }
 
 struct AddConnectionData {
@@ -387,41 +426,58 @@ void Handler::addConnection(NMConnection *connection)
                               userData);
 }
 
-void Handler::addAndActivateConnectionDBus(const NMVariantMapMap &map, const QString &device, const QString &specificObject)
+QCoro::Task<void> Handler::addAndActivateConnectionDBus(const NMVariantMapMap &map, const QString &device, const QString &specificObject)
 {
-    QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::addAndActivateConnection(map, device, specificObject);
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", AddAndActivateConnection);
-    watcher->setProperty("connection", map.value(QStringLiteral("connection")).value(QStringLiteral("id")));
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    const QString name = map.value(QStringLiteral("connection")).value(QStringLiteral("id")).toString();
+    QDBusReply<QDBusObjectPath> reply = co_await NetworkManager::addAndActivateConnection(map, device, specificObject);
+
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToAddConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to add %1", name));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    }
 }
 
 void Handler::deactivateConnection(const QString &connection, const QString &device)
 {
+    deactivateConnectionInternal(connection, device);
+}
+
+QCoro::Task<void> Handler::deactivateConnectionInternal(const QString &_connection, const QString &device)
+{
+    const QString connection = _connection;
     NetworkManager::Connection::Ptr con = NetworkManager::findConnection(connection);
 
     if (!con) {
         qCWarning(PLASMA_NM_LIBS_LOG) << "Not possible to deactivate this connection";
-        return;
+        co_return;
     }
 
-    QDBusPendingReply<> reply;
+    QDBusReply<void> reply;
     for (const NetworkManager::ActiveConnection::Ptr &active : NetworkManager::activeConnections()) {
         if (active->uuid() == con->uuid() && ((!active->devices().isEmpty() && active->devices().first() == device) || active->vpn())) {
             if (active->vpn()) {
-                reply = NetworkManager::deactivateConnection(active->path());
+                reply = co_await NetworkManager::deactivateConnection(active->path());
             } else {
                 NetworkManager::Device::Ptr device = NetworkManager::findNetworkInterface(active->devices().first());
                 if (device) {
-                    reply = device->disconnectInterface();
+                    reply = co_await device->disconnectInterface();
                 }
             }
         }
     }
 
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", Handler::DeactivateConnection);
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToDeactivateConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to deactivate %1", connection));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    }
 }
 
 void Handler::disconnectAll()
@@ -450,22 +506,6 @@ void Handler::enableAirplaneMode(bool enable)
     }
 }
 
-template<typename T>
-void makeDBusCall(const QDBusMessage &message, QObject *context, std::function<void(QDBusPendingReply<T>)> func)
-{
-    QDBusPendingReply<T> reply = QDBusConnection::systemBus().asyncCall(message);
-    auto watcher = new QDBusPendingCallWatcher(reply, context);
-    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, context, [func](QDBusPendingCallWatcher *watcher) {
-        const QDBusPendingReply<T> reply = *watcher;
-        if (!reply.isValid()) {
-            qCWarning(PLASMA_NM_LIBS_LOG) << reply.error().message();
-            return;
-        }
-        func(reply);
-        watcher->deleteLater();
-    });
-}
-
 void setBluetoothEnabled(const QString &path, bool enabled)
 {
     QDBusMessage message =
@@ -478,38 +518,48 @@ void setBluetoothEnabled(const QString &path, bool enabled)
     QDBusConnection::systemBus().asyncCall(message);
 }
 
-void Handler::enableBluetooth(bool enable)
+QCoro::Task<void> Handler::enableBluetooth(bool enable)
 {
     qDBusRegisterMetaType<QMap<QDBusObjectPath, NMVariantMapMap>>();
 
     const QDBusMessage getObjects = QDBusMessage::createMethodCall("org.bluez", "/", "org.freedesktop.DBus.ObjectManager", "GetManagedObjects");
 
-    makeDBusCall<QMap<QDBusObjectPath, NMVariantMapMap>>(getObjects, this, [enable, this](const auto reply) {
-        for (const QDBusObjectPath &path : reply.value().keys()) {
-            const QString objPath = path.path();
-            qCDebug(PLASMA_NM_LIBS_LOG) << "inspecting path" << objPath;
-            const QStringList interfaces = reply.value().value(path).keys();
-            qCDebug(PLASMA_NM_LIBS_LOG) << "interfaces:" << interfaces;
+    QDBusReply<QMap<QDBusObjectPath, NMVariantMapMap>> reply = co_await QDBusConnection::systemBus().asyncCall(getObjects);
 
-            if (!interfaces.contains(QStringLiteral("org.bluez.Adapter1"))) {
-                continue;
-            }
+    if (!reply.isValid()) {
+        qCWarning(PLASMA_NM_LIBS_LOG) << reply.error().message();
+        co_return;
+    }
 
-            // We need to check previous state first
-            if (!enable) {
-                QDBusMessage getPowered = QDBusMessage::createMethodCall("org.bluez", objPath, "org.freedesktop.DBus.Properties", "Get");
-                const QList<QVariant> arguments{QLatin1String("org.bluez.Adapter1"), QLatin1String("Powered")};
-                getPowered.setArguments(arguments);
+    for (const QDBusObjectPath &path : reply.value().keys()) {
+        const QString objPath = path.path();
+        qCDebug(PLASMA_NM_LIBS_LOG) << "inspecting path" << objPath;
+        const QStringList interfaces = reply.value().value(path).keys();
+        qCDebug(PLASMA_NM_LIBS_LOG) << "interfaces:" << interfaces;
 
-                makeDBusCall<QVariant>(getPowered, this, [objPath, this](const auto reply) {
-                    m_bluetoothAdapters.insert(objPath, reply.value().toBool());
-                    setBluetoothEnabled(objPath, false);
-                });
-            } else if (m_bluetoothAdapters.value(objPath)) {
-                setBluetoothEnabled(objPath, true);
-            }
+        if (!interfaces.contains(QStringLiteral("org.bluez.Adapter1"))) {
+            continue;
         }
-    });
+
+        // We need to check previous state first
+        if (!enable) {
+            QDBusMessage getPowered = QDBusMessage::createMethodCall("org.bluez", objPath, "org.freedesktop.DBus.Properties", "Get");
+            const QList<QVariant> arguments{QLatin1String("org.bluez.Adapter1"), QLatin1String("Powered")};
+            getPowered.setArguments(arguments);
+
+            QDBusReply<QVariant> reply = co_await QDBusConnection::systemBus().asyncCall(getPowered);
+
+            if (!reply.isValid()) {
+                qCWarning(PLASMA_NM_LIBS_LOG) << reply.error().message();
+                co_return;
+            }
+
+            m_bluetoothAdapters.insert(objPath, reply.value().toBool());
+            setBluetoothEnabled(objPath, false);
+        } else if (m_bluetoothAdapters.value(objPath)) {
+            setBluetoothEnabled(objPath, true);
+        }
+    }
 }
 
 void Handler::enableNetworking(bool enable)
@@ -528,12 +578,17 @@ void Handler::enableWwan(bool enable)
 }
 
 void Handler::removeConnection(const QString &connection)
+{
+    removeConnectionInternal(connection);
+}
+
+QCoro::Task<void> Handler::removeConnectionInternal(const QString &connection)
 {
     NetworkManager::Connection::Ptr con = NetworkManager::findConnection(connection);
 
     if (!con || con->uuid().isEmpty()) {
         qCWarning(PLASMA_NM_LIBS_LOG) << "Not possible to remove connection " << connection;
-        return;
+        co_return;
     }
 
     // Remove slave connections
@@ -544,23 +599,52 @@ void Handler::removeConnection(const QString &connection)
         }
     }
 
-    QDBusPendingReply<> reply = con->remove();
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", Handler::RemoveConnection);
-    watcher->setProperty("connection", con->name());
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    QDBusReply<void> reply = co_await con->remove();
+
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToRemoveConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to remove %1", con->name()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    } else {
+        KNotification *notification = new KNotification(QStringLiteral("ConnectionRemoved"), KNotification::CloseOnTimeout, this);
+        notification->setText(i18n("Connection %1 has been removed", con->name()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setTitle(con->name());
+        notification->setIconName(QStringLiteral("dialog-information"));
+        notification->sendEvent();
+    }
 }
 
-void Handler::updateConnection(const NetworkManager::Connection::Ptr &connection, const NMVariantMapMap &map)
+QCoro::Task<void> Handler::updateConnection(NetworkManager::Connection::Ptr connection, const NMVariantMapMap &map)
 {
-    QDBusPendingReply<> reply = connection->update(map);
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", UpdateConnection);
-    watcher->setProperty("connection", connection->name());
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+    QDBusReply<void> reply = co_await connection->update(map);
+
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToUpdateConnection"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to update connection %1", connection->name()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    } else {
+        KNotification *notification = new KNotification(QStringLiteral("ConnectionUpdated"), KNotification::CloseOnTimeout, this);
+        notification->setText(i18n("Connection %1 has been updated", connection->name()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setTitle(connection->name());
+        notification->setIconName(QStringLiteral("dialog-information"));
+        notification->sendEvent();
+    }
 }
 
 void Handler::requestScan(const QString &interface)
+{
+    requestScanInternal(interface);
+}
+
+QCoro::Task<void> Handler::requestScanInternal(const QString &interface)
 {
     for (const NetworkManager::Device::Ptr &device : NetworkManager::networkInterfaces()) {
         if (device->type() == NetworkManager::Device::Wifi) {
@@ -587,7 +671,7 @@ void Handler::requestScan(const QString &interface)
                     scheduleRequestScan(wifiDevice->interfaceName(), timeout);
 
                     if (!interface.isEmpty()) {
-                        return;
+                        co_return;
                     }
                     continue;
                 } else if (m_wirelessScanRetryTimer.contains(interface)) {
@@ -596,12 +680,18 @@ void Handler::requestScan(const QString &interface)
                 }
 
                 qCDebug(PLASMA_NM_LIBS_LOG) << "Requesting wifi scan on device" << wifiDevice->interfaceName();
+
                 incrementScansCount();
-                QDBusPendingReply<> reply = wifiDevice->requestScan();
-                auto watcher = new QDBusPendingCallWatcher(reply, this);
-                watcher->setProperty("action", Handler::RequestScan);
-                watcher->setProperty("interface", wifiDevice->interfaceName());
-                connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
+                QDBusReply<void> reply = co_await wifiDevice->requestScan();
+
+                if (!reply.isValid()) {
+                    const QString interface = wifiDevice->interfaceName();
+                    qCWarning(PLASMA_NM_LIBS_LOG) << "Wireless scan on" << interface << "failed:" << reply.error().message();
+                    scanRequestFailed(interface);
+                } else {
+                    qCDebug(PLASMA_NM_LIBS_LOG) << "Wireless scan on" << wifiDevice->interfaceName() << "succeeded";
+                }
+                decrementScansCount();
             }
         }
     }
@@ -628,6 +718,11 @@ void Handler::decrementScansCount()
 }
 
 void Handler::createHotspot()
+{
+    createHotspotInternal();
+}
+
+QCoro::Task<void> Handler::createHotspotInternal()
 {
     bool foundInactive = false;
     bool useApMode = false;
@@ -668,7 +763,7 @@ void Handler::createHotspot()
 
     if (!wifiDev) {
         qCWarning(PLASMA_NM_LIBS_LOG) << "Failed to create hotspot: missing wireless device";
-        return;
+        co_return;
     }
 
     wifiSetting->setInitialized(true);
@@ -706,12 +801,39 @@ void Handler::createHotspot()
     const QVariantMap options = {{QLatin1String("persist"), QLatin1String("volatile")}};
 
     QDBusPendingReply<QDBusObjectPath, QDBusObjectPath, QVariantMap> reply =
-        NetworkManager::addAndActivateConnection2(connectionSettings->toMap(), wifiDev->uni(), QString(), options);
-    auto watcher = new QDBusPendingCallWatcher(reply, this);
-    watcher->setProperty("action", Handler::CreateHotspot);
-    watcher->setProperty("connection", Configuration::self().hotspotName());
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, QOverload<QDBusPendingCallWatcher *>::of(&Handler::hotspotCreated));
+        co_await NetworkManager::addAndActivateConnection2(connectionSettings->toMap(), wifiDev->uni(), QString(), options);
+
+    if (!reply.isValid()) {
+        KNotification *notification = new KNotification(QStringLiteral("FailedToCreateHotspot"), KNotification::CloseOnTimeout, this);
+        notification->setTitle(i18n("Failed to create hotspot %1", Configuration::self().hotspotName()));
+        notification->setComponentName(QStringLiteral("networkmanagement"));
+        notification->setText(reply.error().message());
+        notification->setIconName(QStringLiteral("dialog-warning"));
+        notification->sendEvent();
+    } else {
+        const QString activeConnectionPath = reply.argumentAt(1).value<QDBusObjectPath>().path();
+
+        if (activeConnectionPath.isEmpty()) {
+            co_return;
+        }
+
+        Configuration::self().setHotspotConnectionPath(activeConnectionPath);
+
+        NetworkManager::ActiveConnection::Ptr hotspot = NetworkManager::findActiveConnection(activeConnectionPath);
+
+        if (!hotspot) {
+            co_return;
+        }
+
+        connect(hotspot.data(), &NetworkManager::ActiveConnection::stateChanged, [this](NetworkManager::ActiveConnection::State state) {
+            if (state > NetworkManager::ActiveConnection::Activated) {
+                Configuration::self().setHotspotConnectionPath(QString());
+                Q_EMIT hotspotDisabled();
+            }
+        });
+
+        Q_EMIT hotspotCreated();
+    }
 }
 
 void Handler::stopHotspot()
@@ -825,125 +947,6 @@ void Handler::secretAgentError(const QString &connectionPath, const QString &mes
     Q_EMIT connectionActivationFailed(connectionPath, message);
 }
 
-void Handler::replyFinished(QDBusPendingCallWatcher *watcher)
-{
-    QDBusPendingReply<> reply = *watcher;
-    if (reply.isError() || !reply.isValid()) {
-        KNotification *notification = nullptr;
-        QString error = reply.error().message();
-        Handler::HandlerAction action = (Handler::HandlerAction)watcher->property("action").toUInt();
-        switch (action) {
-        case Handler::ActivateConnection:
-            notification = new KNotification(QStringLiteral("FailedToActivateConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to activate %1", watcher->property("connection").toString()));
-            break;
-        case Handler::AddAndActivateConnection:
-            notification = new KNotification(QStringLiteral("FailedToAddConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to add %1", watcher->property("connection").toString()));
-            break;
-        case Handler::AddConnection:
-            notification = new KNotification(QStringLiteral("FailedToAddConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to add connection %1", watcher->property("connection").toString()));
-            break;
-        case Handler::DeactivateConnection:
-            notification = new KNotification(QStringLiteral("FailedToDeactivateConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to deactivate %1", watcher->property("connection").toString()));
-            break;
-        case Handler::RemoveConnection:
-            notification = new KNotification(QStringLiteral("FailedToRemoveConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to remove %1", watcher->property("connection").toString()));
-            break;
-        case Handler::UpdateConnection:
-            notification = new KNotification(QStringLiteral("FailedToUpdateConnection"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to update connection %1", watcher->property("connection").toString()));
-            break;
-        case Handler::RequestScan: {
-            const QString interface = watcher->property("interface").toString();
-            qCWarning(PLASMA_NM_LIBS_LOG) << "Wireless scan on" << interface << "failed:" << error;
-            scanRequestFailed(interface);
-            decrementScansCount();
-            break;
-        }
-        case Handler::CreateHotspot:
-            notification = new KNotification(QStringLiteral("FailedToCreateHotspot"), KNotification::CloseOnTimeout, this);
-            notification->setTitle(i18n("Failed to create hotspot %1", watcher->property("connection").toString()));
-            break;
-        default:
-            break;
-        }
-
-        if (notification) {
-            notification->setComponentName(QStringLiteral("networkmanagement"));
-            notification->setText(error);
-            notification->setIconName(QStringLiteral("dialog-warning"));
-            notification->sendEvent();
-        }
-    } else {
-        KNotification *notification = nullptr;
-        Handler::HandlerAction action = (Handler::HandlerAction)watcher->property("action").toUInt();
-
-        switch (action) {
-        case Handler::AddConnection:
-            notification = new KNotification(QStringLiteral("ConnectionAdded"), KNotification::CloseOnTimeout, this);
-            notification->setText(i18n("Connection %1 has been added", watcher->property("connection").toString()));
-            break;
-        case Handler::RemoveConnection:
-            notification = new KNotification(QStringLiteral("ConnectionRemoved"), KNotification::CloseOnTimeout, this);
-            notification->setText(i18n("Connection %1 has been removed", watcher->property("connection").toString()));
-            break;
-        case Handler::UpdateConnection:
-            notification = new KNotification(QStringLiteral("ConnectionUpdated"), KNotification::CloseOnTimeout, this);
-            notification->setText(i18n("Connection %1 has been updated", watcher->property("connection").toString()));
-            break;
-        case Handler::RequestScan:
-            qCDebug(PLASMA_NM_LIBS_LOG) << "Wireless scan on" << watcher->property("interface").toString() << "succeeded";
-            decrementScansCount();
-            break;
-        default:
-            break;
-        }
-
-        if (notification) {
-            notification->setComponentName(QStringLiteral("networkmanagement"));
-            notification->setTitle(watcher->property("connection").toString());
-            notification->setIconName(QStringLiteral("dialog-information"));
-            notification->sendEvent();
-        }
-    }
-
-    watcher->deleteLater();
-}
-
-void Handler::hotspotCreated(QDBusPendingCallWatcher *watcher)
-{
-    QDBusPendingReply<QDBusObjectPath, QDBusObjectPath, QVariantMap> reply = *watcher;
-
-    if (!reply.isError() && reply.isValid()) {
-        const QString activeConnectionPath = reply.argumentAt(1).value<QDBusObjectPath>().path();
-
-        if (activeConnectionPath.isEmpty()) {
-            return;
-        }
-
-        Configuration::self().setHotspotConnectionPath(activeConnectionPath);
-
-        NetworkManager::ActiveConnection::Ptr hotspot = NetworkManager::findActiveConnection(activeConnectionPath);
-
-        if (!hotspot) {
-            return;
-        }
-
-        connect(hotspot.data(), &NetworkManager::ActiveConnection::stateChanged, [=](NetworkManager::ActiveConnection::State state) {
-            if (state > NetworkManager::ActiveConnection::Activated) {
-                Configuration::self().setHotspotConnectionPath(QString());
-                Q_EMIT hotspotDisabled();
-            }
-        });
-
-        Q_EMIT hotspotCreated();
-    }
-}
-
 void Handler::primaryConnectionTypeChanged(NetworkManager::ConnectionSettings::ConnectionType type)
 {
     Q_UNUSED(type)
@@ -963,10 +966,10 @@ void Handler::slotRequestWifiCode(QDBusPendingCallWatcher *watcher)
     watcher->deleteLater();
 
     QString ret = watcher->property("ret").toString();
-    const QString connectionName = watcher->property("connectionName").toString();
+    const QString ssid = watcher->property("ssid").toString();
     QDBusPendingReply<NMVariantMapMap> reply = *watcher;
     if (!reply.isValid() || reply.isError()) {
-        Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), connectionName);
+        Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), ssid);
         return;
     }
 
@@ -981,12 +984,14 @@ void Handler::slotRequestWifiCode(QDBusPendingCallWatcher *watcher)
         pass = secret[QStringLiteral("psk")].toString();
         break;
     default:
-        Q_EMIT wifiCodeReceived(QString(), connectionName);
+        Q_EMIT wifiCodeReceived(QString(), ssid);
         return;
     }
     if (!pass.isEmpty()) {
         ret += QStringLiteral("P:") % pass % QLatin1Char(';');
     }
 
-    Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), connectionName);
+    Q_EMIT wifiCodeReceived(ret % QLatin1Char(';'), ssid);
 }
+
+#include "moc_handler.cpp"
