@@ -9,7 +9,7 @@
 #include "networkmodelitem.h"
 #include "plasma_nm_libs.h"
 #include "uiutils.h"
-#include <QVector>
+#include <QList>
 #include <algorithm>
 
 #include <ModemManagerQt/Manager>
@@ -116,7 +116,7 @@ bool NetworkModel::setData(const QModelIndex &index, const QVariant &value, int
         NetworkModelItem *item = m_list.itemAt(row);
         if (item->delayModelUpdates() != delay) {
             item->setDelayModelUpdates(delay);
-            dataChanged(index, index, QVector<int>{DelayModelUpdatesRole});
+            dataChanged(index, index, QList<int>{DelayModelUpdatesRole});
             updateDelayModelUpdates();
             return true;
         }
@@ -126,7 +126,6 @@ bool NetworkModel::setData(const QModelIndex &index, const QVariant &value, int
 
 int NetworkModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
     return parent.isValid() ? 0 : m_list.count();
 }
 
@@ -352,7 +351,7 @@ void NetworkModel::addActiveConnection(const NetworkManager::ActiveConnection::P
         addConnection(connection);
     }
 
-    for (NetworkModelItem *item : m_list.returnItems(NetworkItemsList::NetworkItemsList::Uuid, connection->uuid())) {
+    for (const auto items = m_list.returnItems(NetworkItemsList::NetworkItemsList::Uuid, connection->uuid()); NetworkModelItem * item : items) {
         if (((device && device->uni() == item->devicePath()) || item->devicePath().isEmpty()) || item->type() == NetworkManager::ConnectionSettings::Vpn) {
             item->setActiveConnectionPath(activeConnection->path());
             item->setConnectionState(activeConnection->state());
@@ -371,7 +370,6 @@ void NetworkModel::addActiveConnection(const NetworkManager::ActiveConnection::P
                 }
                 item->setVpnState(state);
             }
-            item->invalidateDetails();
             qCDebug(PLASMA_NM_LIBS_LOG).nospace() << "Item " << item->name() << ": active connection state changed to " << item->connectionState();
 
             if (device && device->uni() == item->devicePath()) {
@@ -1207,3 +1205,5 @@ void NetworkModel::updateFromWirelessNetwork(NetworkModelItem *item,
     item->setSecurityType(securityType);
     updateItem(item);
 }
+
+#include "moc_networkmodel.cpp"
